@@ -76,75 +76,84 @@ class AlgebraicRecon(Recon):
                 else:
                     self.reconLaser.append(theta)
 
-    def plot_MSE(self, isSaving=True):
+    def plot_MSE(self, isSaving=True, log_scale_x=False, log_scale_y=False):
         """
         Plot the Mean Squared Error (MSE) of the reconstruction.
-        
+
         Parameters:
-            MSE: list of float, Mean Squared Error values for each iteration
-                      If None, uses the MSE from self.MSE.
-        
+            isSaving: bool, whether to save the plot.
+            log_scale_x: bool, if True, use logarithmic scale for the x-axis.
+            log_scale_y: bool, if True, use logarithmic scale for the y-axis.
         Returns:
             None
         """
         if not self.MSE:
             raise ValueError("MSE is empty. Please calculate MSE first.")
-        
-        best_idx = self.indices[np.argmin(self.MSE)]
-        
-        print(f"Lowest MSE = {np.min(self.MSE):.4f} at iteration {best_idx+1}")
 
+        best_idx = self.indices[np.argmin(self.MSE)]
+
+        print(f"Lowest MSE = {np.min(self.MSE):.4f} at iteration {best_idx+1}")
         # Plot MSE curve
         plt.figure(figsize=(7, 5))
-        plt.plot(self.indices,self.MSE, 'r-', label="MSE curve")
-
+        plt.plot(self.indices, self.MSE, 'r-', label="MSE curve")
         # Add blue dashed lines
         plt.axhline(np.min(self.MSE), color='blue', linestyle='--', label=f"Min MSE = {np.min(self.MSE):.4f}")
         plt.axvline(best_idx, color='blue', linestyle='--', label=f"Iteration = {best_idx+1}")
-
         plt.xlabel("Iteration")
         plt.ylabel("MSE")
         plt.title("MSE vs. Iteration")
+        if log_scale_x:
+            plt.xscale('log')
+        if log_scale_y:
+            plt.yscale('log')
         plt.legend()
-        plt.grid(True)
+        plt.grid(True, which="both", ls="-")
         plt.tight_layout()
-
         if isSaving and self.saveDir is not None:
-            now = datetime.now()    
+            now = datetime.now()
             date_str = now.strftime("%Y_%d_%m_%y")
+            scale_str = ""
+            if log_scale_x and log_scale_y:
+                scale_str = "_loglog"
+            elif log_scale_x:
+                scale_str = "_logx"
+            elif log_scale_y:
+                scale_str = "_logy"
             if self.optimizer == OptimizerType.MLEM:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_MLEM{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_MLEM{scale_str}{date_str}.png')
             elif self.optimizer == OptimizerType.LS:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS{scale_str}{date_str}.png')
             elif self.optimizer == OptimizerType.LS_TV:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_MSE_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}{scale_str}{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"MSE plot saved to {SavingFolder}")
-        
+
         plt.show()
 
     def show_MSE_bestRecon(self, isSaving=True):
         if not self.MSE:
             raise ValueError("MSE is empty. Please calculate MSE first.")
 
-        best_idx = self.indices[np.argmin(self.MSE)]
+
+        best_idx = np.argmin(self.MSE)
+        print(best_idx)
         best_recon = self.reconPhantom[best_idx]
 
         # Crée la figure et les axes
         fig, axs = plt.subplots(1, 3, figsize=(15, 5))
 
         # Left: Best reconstructed image (normalized)
-        im0 = axs[0].imshow(best_recon / np.max(best_recon),
+        im0 = axs[0].imshow(best_recon,
                             extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
                                     self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
-        axs[0].set_title(f"Min MSE Reconstruction\nIter {best_idx}, MSE={np.min(self.MSE):.4f}")
+        axs[0].set_title(f"Min MSE Reconstruction\nIter {self.indices[best_idx]}, MSE={np.min(self.MSE):.4f}")
         axs[0].set_xlabel("x (mm)", fontsize=12)
         axs[0].set_ylabel("z (mm)", fontsize=12)
         axs[0].tick_params(axis='both', which='major', labelsize=8)
 
         # Middle: Ground truth (normalized)
-        im1 = axs[1].imshow(self.experiment.OpticImage.phantom / np.max(self.experiment.OpticImage.phantom),
+        im1 = axs[1].imshow(self.experiment.OpticImage.phantom,
                             extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
                                     self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
@@ -156,9 +165,10 @@ class AlgebraicRecon(Recon):
 
         # Right: Reconstruction at last iteration
         lastRecon = self.reconPhantom[-1]
+        print(lastRecon.shape)
         if self.experiment.OpticImage.phantom.shape != lastRecon.shape:
             lastRecon = lastRecon.T
-        im2 = axs[2].imshow(lastRecon / np.max(lastRecon),
+        im2 = axs[2].imshow(lastRecon,
                             extent=(self.experiment.params.general['Xrange'][0]*1000, self.experiment.params.general['Xrange'][1]*1000,
                                     self.experiment.params.general['Zrange'][1]*1000, self.experiment.params.general['Zrange'][0]*1000),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
@@ -194,98 +204,156 @@ class AlgebraicRecon(Recon):
 
         plt.show()
 
-    def show_theta_animation(self, vmin=None, vmax=None, duration=5000, save_path=None):
+    def show_theta_animation(self, vmin=None, vmax=None, total_duration_ms=3000, save_path=None, max_frames=1000, isPropMSE=True):
         """
-        Show theta iteration animation (for Jupyter) and optionally save it as a GIF.
-        
+        Show theta iteration animation with speed proportional to MSE acceleration.
+        In "propMSE" mode: slow down when MSE changes rapidly, speed up when MSE stagnates.
+
         Parameters:
-            matrix_theta: list of (z, x) ndarray, Algebraic reconstructions
-            x: 1D array, x-coordinates (in meters)
-            z: 1D array, z-coordinates (in meters)
             vmin, vmax: color limits (optional)
-            duration: duration of the animation in milliseconds
-            save_path: path to save animation (e.g., 'theta.gif' or 'theta.mp4')
+            total_duration_ms: total duration of the animation in milliseconds
+            save_path: path to save animation (e.g., 'theta.gif')
+            max_frames: maximum number of frames to include (default: 1000)
+            isPropMSE: if True, use adaptive speed based on MSE (default: True)
         """
-        if len(self.reconPhantom) == 0 or len(self.reconPhantom) == 1:
-            raise ValueError("No theta matrix available for animation.")
+        import matplotlib as mpl
+        mpl.rcParams['animation.embed_limit'] = 200
+
+        if len(self.reconPhantom) == 0 or len(self.reconPhantom) < 2:
+            raise ValueError("Not enough theta matrices available for animation.")
+
+        if isPropMSE and (self.MSE is None or len(self.MSE) == 0):
+            raise ValueError("MSE is empty or not calculated. Please calculate MSE first.")
 
         frames = np.array(self.reconPhantom)
-        num_frames = len(frames)
+        mse = np.array(self.MSE)
 
-        interval = max(1, int(duration / num_frames))
+        # Sous-échantillonnage initial
+        step = max(1, len(frames) // max_frames)
+        frames_subset = frames[::step]
+        indices_subset = self.indices[::step]
+        mse_subset = mse[::step]
 
         if vmin is None:
-            vmin = np.min(frames)
+            vmin = np.min(frames_subset)
         if vmax is None:
-            vmax = np.max(frames)
+            vmax = np.max(frames_subset)
 
-        fig, ax = plt.subplots(figsize=(5, 5))
-        im = ax.imshow(frames[0],
-                    extent=(self.experiment.params.general['Xrange'][0],self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
-                    vmin=vmin, vmax=vmax,
-                    aspect='equal', cmap='hot')
-
-        title = ax.set_title("Iteration 0")
+        fig, ax = plt.subplots(figsize=(4, 4), dpi=100)
+        im = ax.imshow(
+            frames_subset[0],
+            extent=(
+                self.experiment.params.general['Xrange'][0],
+                self.experiment.params.general['Xrange'][1],
+                self.experiment.params.general['Zrange'][1],
+                self.experiment.params.general['Zrange'][0]
+            ),
+            vmin=vmin,
+            vmax=vmax,
+            aspect='equal',
+            cmap='hot'
+        )
+        title = ax.set_title(f"Iteration {indices_subset[0]}")
         ax.set_xlabel("x (mm)")
         ax.set_ylabel("z (mm)")
         plt.tight_layout()
 
+        if isPropMSE:
+            # Calcule la dérivée première (variation du MSE)
+            mse_diff = np.gradient(mse_subset)
+            # Calcule la dérivée seconde (accélération du MSE)
+            mse_accel = np.gradient(mse_diff)
+            # Normalise l'accélération entre 0 et 1 (en valeur absolue)
+            mse_accel_normalized = np.abs(mse_accel)
+            mse_accel_normalized /= (np.max(mse_accel_normalized) + 1e-10)
+
+            # Prépare les frames pour le mode "propMSE"
+            all_frames = []
+            all_indices = []
+
+            for i in range(len(frames_subset)):
+                # Nombre de duplications inversement proportionnel à l'accélération (pour ralentir quand MSE change vite)
+                # Plus l'accélération est élevée, plus on duplique (pour ralentir)
+                num_duplicates = max(1, int(1 + 9 * mse_accel_normalized[i]))
+                all_frames.extend([frames_subset[i]] * num_duplicates)
+                all_indices.extend([indices_subset[i]] * num_duplicates)
+
+            # Ajuste le nombre total de frames pour respecter la durée
+            target_frames = int(total_duration_ms / 10)  # 10 ms par frame
+            if len(all_frames) > target_frames:
+                step_prop = len(all_frames) // target_frames
+                all_frames = all_frames[::step_prop]
+                all_indices = all_indices[::step_prop]
+
+        else:  # Mode "linéaire"
+            all_frames = frames_subset
+            all_indices = indices_subset
+
         def update(frame_idx):
-            im.set_array(frames[frame_idx])
-            title.set_text(f"Iteration {self.indices[frame_idx]}")
+            im.set_array(all_frames[frame_idx])
+            title.set_text(f"Iteration {all_indices[frame_idx]}")
             return [im, title]
 
-        ani = animation.FuncAnimation(fig, update, frames=num_frames, interval=interval, blit=False)
+        ani = animation.FuncAnimation(
+            fig,
+            update,
+            frames=len(all_frames),
+            interval=10,  # 10 ms par frame
+            blit=False,
+        )
 
         if save_path:
             if save_path.endswith(".gif"):
-                ani.save(save_path, writer="pillow", fps=1000 // interval)
+                ani.save(save_path, writer=animation.PillowWriter(fps=100))
             elif save_path.endswith(".mp4"):
-                ani.save(save_path, writer="ffmpeg", fps=1000 // interval)
-            else:
-                raise ValueError("Unsupported file format. Use .gif or .mp4")
+                ani.save(save_path, writer="ffmpeg", fps=30)
             print(f"Animation saved to {save_path}")
 
         plt.close(fig)
-        plt.rcParams["animation.html"] = "jshtml"
-        return HTML(ani.to_jshtml())  
-
-    def plot_SSIM(self, isSaving=True):
+        return HTML(ani.to_jshtml())
 
+    def plot_SSIM(self, isSaving=True, log_scale_x=False, log_scale_y=False):
         if not self.SSIM:
             raise ValueError("SSIM is empty. Please calculate SSIM first.")
-        
-        best_idx = self.indices[np.argmax(self.SSIM)]
-        
-        print(f"Highest SSIM = {np.max(self.SSIM):.4f} at iteration {best_idx+1}")
 
+        best_idx = self.indices[np.argmax(self.SSIM)]
+
+        print(f"Highest SSIM = {np.max(self.SSIM):.4f} at iteration {best_idx+1}")
         # Plot SSIM curve
         plt.figure(figsize=(7, 5))
-        plt.plot(self.indices,self.SSIM, 'r-', label="SSIM curve")
-
+        plt.plot(self.indices, self.SSIM, 'r-', label="SSIM curve")
         # Add blue dashed lines
         plt.axhline(np.max(self.SSIM), color='blue', linestyle='--', label=f"Max SSIM = {np.max(self.SSIM):.4f}")
         plt.axvline(best_idx, color='blue', linestyle='--', label=f"Iteration = {best_idx}")
-
         plt.xlabel("Iteration")
         plt.ylabel("SSIM")
         plt.title("SSIM vs. Iteration")
+        if log_scale_x:
+            plt.xscale('log')
+        if log_scale_y:
+            plt.yscale('log')
         plt.legend()
-        plt.grid(True)
+        plt.grid(True, which="both", ls="-")
         plt.tight_layout()
-
         if isSaving and self.saveDir is not None:
-            now = datetime.now()    
+            now = datetime.now()
             date_str = now.strftime("%Y_%d_%m_%y")
+            scale_str = ""
+            if log_scale_x and log_scale_y:
+                scale_str = "_loglog"
+            elif log_scale_x:
+                scale_str = "_logx"
+            elif log_scale_y:
+                scale_str = "_logy"
             if self.optimizer == OptimizerType.MLEM:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_MLEM{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_MLEM{scale_str}{date_str}.png')
             elif self.optimizer == OptimizerType.LS:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS{scale_str}{date_str}.png')
             elif self.optimizer == OptimizerType.LS_TV:
-                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}_{date_str}.png')
+                SavingFolder = os.path.join(self.saveDir, f'{self.SMatrix.shape[3]}_SCANS_SSIM_plot_LS_TV_Lambda_{self.lambda_reg}_LFactor_{self.L_Factor}{scale_str}{date_str}.png')
             plt.savefig(SavingFolder, dpi=300)
             print(f"SSIM plot saved to {SavingFolder}")
-        
+
         plt.show()
 
     def show_SSIM_bestRecon(self, isSaving=True):
@@ -293,7 +361,7 @@ class AlgebraicRecon(Recon):
         if not self.SSIM:
             raise ValueError("SSIM is empty. Please calculate SSIM first.")
 
-        best_idx = self.indices[np.argmax(self.SSIM)]
+        best_idx = np.argmax(self.SSIM)
         best_recon = self.reconPhantom[best_idx]
 
         # ----------------- Plotting -----------------
@@ -303,16 +371,16 @@ class AlgebraicRecon(Recon):
         lambda_max = np.max(self.experiment.OpticImage.laser.intensity)
 
         # Left: Best reconstructed image (normalized)
-        im0 = axs[0].imshow(best_recon / lambda_max, 
+        im0 = axs[0].imshow(best_recon, 
                             extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
-        axs[0].set_title(f"Max SSIM Reconstruction\nIter {best_idx}, SSIM={np.min(self.MSE):.4f}")
+        axs[0].set_title(f"Max SSIM Reconstruction\nIter {self.indices[best_idx]}, SSIM={np.min(self.MSE):.4f}")
         axs[0].set_xlabel("x (mm)")
         axs[0].set_ylabel("z (mm)")
         plt.colorbar(im0, ax=axs[0])
 
         # Middle: Ground truth (normalized)
-        im1 = axs[1].imshow(self.experiment.OpticImage.laser.intensity / lambda_max, 
+        im1 = axs[1].imshow(self.experiment.OpticImage.laser.intensity, 
                             extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
         axs[1].set_title(r"Ground Truth ($\lambda$)")
@@ -322,7 +390,7 @@ class AlgebraicRecon(Recon):
 
         # Right: Reconstruction at iter 350
         lastRecon = self.reconPhantom[-1] 
-        im2 = axs[2].imshow(lastRecon / lambda_max,
+        im2 = axs[2].imshow(lastRecon,
                             extent=(self.experiment.params.general['Xrange'][0], self.experiment.params.general['Xrange'][1], self.experiment.params.general['Zrange'][1], self.experiment.params.general['Zrange'][0]),
                             cmap='hot', aspect='equal', vmin=0, vmax=1)
         axs[2].set_title(f"Last Reconstruction\nIter {self.numIterations * self.numSubsets}, SSIM={self.SSIM[-1]:.4f}")
@@ -391,156 +459,159 @@ class AlgebraicRecon(Recon):
             print(f"Figure saved to: {save_path}")
         plt.show()
         
-    def show_reconstruction_progress(self, start=0, fin=None, duration=5000, save_path=None):
+    def show_reconstruction_progress(self, start=0, fin=None, save_path=None, with_tumor=True):
         """
-        Show the reconstruction progress for both with and without tumor.
+        Show the reconstruction progress for either with or without tumor.
+        If isPropMSE is True, the frame selection is adapted to MSE changes.
+        Otherwise, indices are evenly spaced between start and fin.
+
         Parameters:
             start: int, starting iteration index
             fin: int, ending iteration index (inclusive)
             duration: int, duration of the animation in milliseconds
             save_path: str, path to save the figure (optional)
+            with_tumor: bool, if True, show reconstruction with tumor; else without (default: True)
+            isPropMSE: bool, if True, use adaptive speed based on MSE (default: True)
         """
-
-        if self.reconLaser is None or self.reconLaser == []:
-            raise ValueError("Reconstructed laser is empty. Run reconstruction first.")
-        if isinstance(self.Laser,list) and len(self.Laser) == 1:
-            raise ValueError("Reconstructed Image without tumor is a single frame. Run reconstruction with isSavingEachIteration=True to get a sequence of frames.")
-        if self.reconPhantom is None or self.reconPhantom == []:
-            raise ValueError("Reconstructed phantom is empty. Run reconstruction first.")
-        if isinstance(self.reconPhantom, list) and len(self.reconPhantom) == 1:
-            raise ValueError("Reconstructed Image with tumor is a single frame. Run reconstruction with isSavingEachIteration=True to get a sequence of frames.")
-        
-        step = 1 / self.experiment.params.general['f_saving']* duration / 1000  # convert duration to seconds and divide by frame rate
+        import matplotlib as mpl
+        mpl.rcParams['animation.embed_limit'] = 200
 
         if fin is None:
-            fin = len(self.reconPhantom) - 1
+            fin = len(self.reconPhantom) - 1 if with_tumor else len(self.reconLaser) - 1
 
-        iter_list = self.indices
-        nrows = len(iter_list)
-        ncols = 3  # Recon, |Recon - GT|, Ground Truth
+        # Check data availability
+        if with_tumor:
+            if self.reconPhantom is None or self.reconPhantom == []:
+                raise ValueError("Reconstructed phantom is empty. Run reconstruction first.")
+            if isinstance(self.reconPhantom, list) and len(self.reconPhantom) == 1:
+                raise ValueError("Reconstructed Image with tumor is a single frame. Run reconstruction with isSavingEachIteration=True.")
+            recon_list = self.reconPhantom
+            ground_truth = self.experiment.OpticImage.phantom
+            title_suffix = "with_tumor"
+        else:
+            if self.reconLaser is None or self.reconLaser == []:
+                raise ValueError("Reconstructed laser is empty. Run reconstruction first.")
+            if isinstance(self.reconLaser, list) and len(self.reconLaser) == 1:
+                raise ValueError("Reconstructed Image without tumor is a single frame. Run reconstruction with isSavingEachIteration=True.")
+            recon_list = self.reconLaser
+            ground_truth = self.experiment.OpticImage.laser.intensity
+            title_suffix = "without_tumor"
 
-        vmin = 0
-        vmax = 1
-
-        recon_without_tumor_list = []
-        diff_abs_without_tumor_list = []
-        mse_without_tumor_list = []
+        # Collect data for all iterations
+        recon_list_data = []
+        diff_abs_list = []
+        mse_list = []
         noise_list = []
 
-        for i in iter_list:
-            
-            diff_abs_without_tumor = np.abs(recon_without_tumor - self.experiment.OpticImage.laser.intensity)
-            mse_without_tumor = mse(self.experiment.OpticImage.laser.intensity.flatten(), recon_without_tumor.flatten())
+        for i in range(start, fin + 1):
+            recon = recon_list[i]
+            diff_abs = np.abs(recon - ground_truth)
+            mse = np.mean((ground_truth.flatten() - recon.flatten())**2)
+            noise = np.mean(np.abs(recon - ground_truth))
 
-            noise = np.mean(np.abs(self.reconLaser[i] - self.experiment.OpticImage.laser.intensity))
-
-            recon_without_tumor_list.append(recon_without_tumor)
-            diff_abs_without_tumor_list.append(diff_abs_without_tumor)
-            mse_without_tumor_list.append(mse_without_tumor)
+            recon_list_data.append(recon)
+            diff_abs_list.append(diff_abs)
+            mse_list.append(mse)
             noise_list.append(noise)
 
-        global_min_diff_abs_without_tumor = np.min([d.min() for d in diff_abs_without_tumor_list[1:]])
-        global_max_diff_abs_without_tumor = np.max([d.max() for d in diff_abs_without_tumor_list[1:]])
+        # Calculate global min/max for difference images
+        global_min_diff = np.min([d.min() for d in diff_abs_list[1:]])
+        global_max_diff = np.max([d.max() for d in diff_abs_list[1:]])
+
+        # Evenly spaced indices
+        num_frames = min(5, fin - start + 1)
+        all_indices = np.linspace(start, fin, num_frames, dtype=int).tolist()
+
+        # Plot
+        nrows = min(5, len(all_indices))
+        ncols = 3  # Recon, |Recon - GT|, Ground Truth
+        vmin, vmax = 0, 1
 
         fig, axs = plt.subplots(nrows=nrows, ncols=ncols, figsize=(12, 3 * nrows))
 
-        for i, iter_idx in enumerate(iter_list):
-            recon_without_tumor = recon_without_tumor_list[i]
-            diff_abs_without_tumor = diff_abs_without_tumor_list[i]
-            mse_without_tumor = mse_without_tumor_list[i]
-            noise = noise_list[i]
+        for i, iter_idx in enumerate(all_indices[:nrows]):
+            idx_in_list = iter_idx - start  # Index in the collected data lists
+            recon = recon_list_data[idx_in_list]
+            diff_abs = diff_abs_list[idx_in_list]
+            mse_val = mse_list[idx_in_list]
+            noise = noise_list[idx_in_list]
 
-            im0 = axs[i, 0].imshow(recon_without_tumor, cmap='hot', vmin=vmin, vmax=vmax, aspect='equal')
-            axs[i, 0].set_title(f"Reconstruction\nIter {iter_idx}, MSE={mse_without_tumor:.2e}", fontsize=10)
+            im0 = axs[i, 0].imshow(recon, cmap='hot', vmin=vmin, vmax=vmax, aspect='equal')
+            axs[i, 0].set_title(f"Reconstruction\nIter {self.indices[iter_idx]}, MSE={mse_val:.2e}", fontsize=10)
             axs[i, 0].axis('off')
             plt.colorbar(im0, ax=axs[i, 0])
 
-            if i >= 0 :
-                im1 = axs[i, 1].imshow(diff_abs_without_tumor, cmap='viridis', vmin=np.min(diff_abs_without_tumor), vmax=np.max(diff_abs_without_tumor), aspect='equal')
-            else :
-                im1 = axs[i, 1].imshow(diff_abs_without_tumor, cmap='viridis', vmin=global_min_diff_abs_without_tumor, vmax=global_max_diff_abs_without_tumor, aspect='equal')
+            im1 = axs[i, 1].imshow(diff_abs, cmap='viridis',
+                                vmin=global_min_diff,
+                                vmax=global_max_diff,
+                                aspect='equal')
             axs[i, 1].set_title(f"|Recon - Ground Truth|\nNoise={noise:.2e}", fontsize=10)
             axs[i, 1].axis('off')
             plt.colorbar(im1, ax=axs[i, 1])
 
-            # 右图：ground truth
-            im2 = axs[i, 2].imshow(self.experiment.OpticImage.laser.intensity, cmap='hot', vmin=vmin, vmax=vmax, aspect='equal')
-            axs[i, 2].set_title(r"Ground Truth ($\lambda$)", fontsize=10)
+            im2 = axs[i, 2].imshow(ground_truth, cmap='hot', vmin=vmin, vmax=vmax, aspect='equal')
+            axs[i, 2].set_title(r"Ground Truth", fontsize=10)
             axs[i, 2].axis('off')
             plt.colorbar(im2, ax=axs[i, 2])
 
         plt.tight_layout()
+
         if save_path:
+            # Add suffix to filename based on with_tumor parameter
+            if '.' in save_path:
+                name, ext = save_path.rsplit('.', 1)
+                save_path = f"{name}_{title_suffix}.{ext}"
+            else:
+                save_path = f"{save_path}_{title_suffix}"
             plt.savefig(save_path, dpi=300)
             print(f"Figure saved to: {save_path}")
+
         plt.show()
 
-        # with tumor
+    def save(self, withTumor=True):
+        """
+        Save the reconstruction results (reconPhantom is with tumor, reconLaser is without tumor) and indices of the saved recon results, in format numpy.
+        Warnings : reconPhantom and reconLaser are lists of 2D numpy arrays, each array corresponding to one iteration.
+        """
+        if self.saveDir is None:
+            raise ValueError("Save directory is not specified. Please set saveDir before saving.")
+        if withTumor:
+            if not self.reconPhantom or len(self.reconPhantom) == 0:
+                raise ValueError("Reconstructed phantom is empty. Run reconstruction first.")
+            np.save(os.path.join(self.saveDir, 'results', 'reconPhantom.npy'), np.array(self.reconPhantom))
+        else:
+            if not self.reconLaser or len(self.reconLaser) == 0:
+                raise ValueError("Reconstructed laser is empty. Run reconstruction first.")
+            np.save(os.path.join(self.saveDir, 'results', 'reconLaser.npy'), np.array(self.reconLaser))
+        np.save(os.path.join(self.saveDir, 'results', 'reconIndices.npy'), np.array(self.indices))
+        print(f"Reconstruction results saved to {os.path.join(self.saveDir, 'results')}")
 
-        if fin is None:
-            fin = len(self.reconPhantom) - 1
+    def load(self, withTumor=True):
+        """
+        Load the reconstruction results (reconPhantom is with tumor, reconLaser is without tumor) and indices of the saved recon results, in format numpy.
+        Warnings : reconPhantom and reconLaser are lists of 2D numpy arrays, each array corresponding to one iteration.
+        """
+        if self.saveDir is None:
+            raise ValueError("Save directory is not specified. Please set saveDir before loading.")
+        if withTumor:
+            recon_path = os.path.join(self.saveDir, 'results', 'reconPhantom.npy')
+            if not os.path.isfile(recon_path):
+                raise FileNotFoundError(f"Reconstructed phantom file not found: {recon_path}")
+            self.reconPhantom = list(np.load(recon_path, allow_pickle=True))
+        else:
+            recon_path = os.path.join(self.saveDir, 'results', 'reconLaser.npy')
+            if not os.path.isfile(recon_path):
+                raise FileNotFoundError(f"Reconstructed laser file not found: {recon_path}")
+            self.reconLaser = list(np.load(recon_path, allow_pickle=True))
+        indices_path = os.path.join(self.saveDir, 'results', 'reconIndices.npy')
+        if not os.path.isfile(indices_path):
+            raise FileNotFoundError(f"Reconstruction indices file not found: {indices_path}")
+        self.indices = list(np.load(indices_path, allow_pickle=True))
+        print(f"Reconstruction results loaded from {os.path.join(self.saveDir, 'results')}")
 
-        iter_list = list(range(start, fin + 1, step))
-        nrows = len(iter_list)
-        ncols = 3  # Recon, |Recon - GT|, Ground Truth
+    def normalizeSMatrix(self):
+        self.SMatrix = self.SMatrix / (self.params.acoustic['voltage']*self.params.acoustic['sensitivity'])
 
-        vmin = 0
-        vmax = 1
-
-        recon_with_tumor_list = []
-        diff_abs_with_tumor_list = []
-        mse_with_tumor_list = []
-        noise_list = []
-
-        for i in iter_list:
-            recon_with_tumor = self.reconPhantom[i]
-            diff_abs_with_tumor = np.abs(recon_with_tumor - self.experiment.OpticImage.phantom)
-            mse_with_tumor = mse(self.experiment.OpticImage.phantom.flatten(), recon_with_tumor.flatten())
-
-            noise = np.mean(np.abs(self.reconPhantom[i] - self.experiment.OpticImage.phantom))  # !! without tumor
-
-            recon_with_tumor_list.append(recon_with_tumor)
-            diff_abs_with_tumor_list.append(diff_abs_with_tumor)
-            mse_with_tumor_list.append(mse_with_tumor)  
-            noise_list.append(noise)
-
-
-        global_min_diff_abs_with_tumor = np.min([d.min() for d in diff_abs_with_tumor_list[1:]])
-        global_max_diff_abs_with_tumor = np.max([d.max() for d in diff_abs_with_tumor_list[1:]])
-
-        _, axs = plt.subplots(nrows=nrows, ncols=ncols, figsize=(12, 3 * nrows))
-
-        for i, iter_idx in enumerate(iter_list):
-            recon_with_tumor = recon_with_tumor_list[i]
-            diff_abs_with_tumor = diff_abs_with_tumor_list[i]
-            mse_with_tumor = mse_with_tumor_list[i]
-            noise = noise_list[i]
-
-            im0 = axs[i, 0].imshow(recon_with_tumor, cmap='hot', vmin=vmin, vmax=vmax, aspect='equal')
-            axs[i, 0].set_title(f"Reconstruction\nIter {iter_idx}, MSE={mse_with_tumor:.2e}", fontsize=10)
-            axs[i, 0].axis('off')
-            plt.colorbar(im0, ax=axs[i, 0])
-
-
-            if i >= 0 :
-                im1 = axs[i, 1].imshow(diff_abs_with_tumor, cmap='viridis', vmin=np.min(diff_abs_with_tumor), vmax=np.max(diff_abs_with_tumor), aspect='equal')
-            else :
-                im1 = axs[i, 1].imshow(diff_abs_with_tumor, cmap='viridis', vmin=global_min_diff_abs_with_tumor, vmax=global_max_diff_abs_with_tumor, aspect='equal')
-            axs[i, 1].set_title(f"|Recon - Ground Truth|\nNoise={noise:.2e}", fontsize=10)
-            axs[i, 1].axis('off')
-            plt.colorbar(im1, ax=axs[i, 1])
-
-            im2 = axs[i, 2].imshow(self.experiment.OpticImage.phantom, cmap='hot', vmin=vmin, vmax=vmax, aspect='equal')
-            axs[i, 2].set_title(r"Ground Truth ($\lambda$)", fontsize=10)
-            axs[i, 2].axis('off')
-            plt.colorbar(im2, ax=axs[i, 2])
-
-        plt.tight_layout()
-        if save_path:
-            plt.savefig(save_path, dpi=300)
-            print(f"Figure saved to: {save_path}")
-        plt.show()
-   
     # PRIVATE METHODS
 
     def _AlgebraicReconPython(self,withTumor):
@@ -707,4 +778,58 @@ class AlgebraicRecon(Recon):
         else:
             warnings.warn("Insufficient GPU memory for single GPU LS with TV regularization. Falling back to CPU.")
             raise NotImplementedError("LS with TV regularization is not implemented for CPU.")
-        return result, indices
\ No newline at end of file
+        return result, indices
+    
+    # STATIC METHODS
+    @staticmethod
+    def plot_mse_comparison(recon_list, labels=None):
+        """
+        Affiche les courbes de MSE pour chaque reconstruction dans recon_list.
+
+        Args:
+            recon_list (list): Liste d'objets recon (doivent avoir les attributs 'indices' et 'MSE').
+            labels (list, optional): Liste des labels pour chaque courbe. Si None, utilise "Recon i".
+        """
+        if labels is None:
+            labels = [f"Recon {i+1}" for i in range(len(recon_list))]
+
+        plt.figure(figsize=(4.5, 3.5))
+        colors = ['red', 'green', 'blue', 'orange', 'purple']  # Ajoute d'autres couleurs si nécessaire
+
+        for i, recon in enumerate(recon_list):
+            color = colors[i % len(colors)]
+            label = labels[i] if i < len(labels) else f"Recon {i+1}"
+
+            # Trouve l'index et la valeur minimale du MSE
+            best_idx = recon.indices[np.argmin(recon.MSE)]
+            min_mse = np.min(recon.MSE)
+
+            # Trace la courbe de MSE
+            plt.plot(recon.indices, recon.MSE, f'{color}-', label=label)
+            # Ligne horizontale pour le min MSE
+            plt.axhline(min_mse, color=color, linestyle='--', alpha=0.5)
+            # Ligne verticale pour l'itération du min MSE
+            plt.axvline(best_idx, color=color, linestyle='--', alpha=0.5)
+
+        plt.xlabel("Iteration")
+        plt.ylabel("MSE")
+        plt.title("MSE vs. Iteration (Comparison)")
+        plt.xscale('log')
+        plt.yscale('log')
+        plt.grid(True, which="both", ls="-")
+
+        # Légende personnalisée
+        handles = []
+        for i, recon in enumerate(recon_list):
+            color = colors[i % len(colors)]
+            best_idx = recon.indices[np.argmin(recon.MSE)]
+            min_mse = np.min(recon.MSE)
+            handles.append(
+                plt.Line2D([0], [0], color=color,
+                        label=f"{labels[i] if labels and i < len(labels) else f'Recon {i+1}'} (min={min_mse:.4f} @ it.{best_idx+1})")
+            )
+
+        plt.legend(handles=handles, loc='upper right')
+        plt.tight_layout()
+        plt.show()
+
