@@ -115,7 +115,7 @@ class Recon(ABC):
                 image = self.reconPhantom
             # Phantom original
             im0 = axs[0].imshow(
-                self.experiment.OpticImage.phantom/np.max(self.experiment.OpticImage.phantom),
+                self.experiment.OpticImage.phantom,
                 cmap='hot',
                 vmin=0,
                 vmax=1,
@@ -133,7 +133,7 @@ class Recon(ABC):
             axs[0].tick_params(axis='both', which='major', labelsize=8)
             # Phantom reconstruit
             im1 = axs[1].imshow(
-                image/np.max(image),
+                image,
                 cmap='hot',
                 vmin=0,
                 vmax=1,
